@@ -1,11 +1,12 @@
 /*
-    SPDX-FileCopyrightText: 2023 Laurent Montel <montel@kde.org>
+    SPDX-FileCopyrightText: 2023-2024 Laurent Montel <montel.org>
 
     SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include "manualconfiguration.h"
 #include "accountwizard_debug.h"
+#include "servertest.h"
 #include <KLocalizedString>
 #include <QRegularExpression>
 #include <QUrl>
@@ -79,17 +80,25 @@ Resource::ResourceInfo ManualConfiguration::createPop3Resource() const
     return info;
 }
 
-QString ManualConfiguration::convertIncomingAuthenticationProtocol(int index) const
+QString ManualConfiguration::convertIncomingAuthenticationProtocol(int protocol) const
 {
-    switch (index) {
-    case 0:
-        return QStringLiteral("STARTTLS");
-    case 1:
-        return QStringLiteral("SSL");
-    case 2:
-        return QStringLiteral("NONE");
+    switch (protocol) {
+    case 0: // Clear Text
+        return QStringLiteral("clear");
+    case 1: // LOGIN
+        return QStringLiteral("login");
+    case 2: // PLAIN
+        return QStringLiteral("plain");
+    case 3: // CRAM-MD5
+        return QStringLiteral("cram-md5");
+    case 4: // DIGEST-MD5
+        return QStringLiteral("digest-md5");
+    case 5: // NTLM
+        return QStringLiteral("ntlm");
+    case 6: // GSSAPI
+        return QStringLiteral("gssapi");
     }
-    Q_UNREACHABLE();
+    qCWarning(ACCOUNTWIZARD_LOG) << " Impossible to convert protocol: " << protocol;
     return {};
 }
 
@@ -124,29 +133,6 @@ Resource::ResourceInfo ManualConfiguration::createImapResource() const
     settings.insert(QStringLiteral("AccountIdentity"), mIdentityId);
     settings.insert(QStringLiteral("Authentication"), convertIncomingAuthenticationProtocol(mCurrentIncomingAuthenticationProtocol));
     settings.insert(QStringLiteral("Safety"), convertIncomingSecurityProtocol(mCurrentIncomingSecurityProtocol));
-
-    // if ( server == "imap.gmail.com" ) {
-    //     imapRes.setOption( "Authentication", 9 ); // XOAuth2
-    //     arg = "ssl";
-    // } else {
-    //     imapRes.setOption( "Authentication", 7 ); // ClearText
-    // }
-    // if ( arg == "ssl" ) {
-    //   // The ENUM used for authentication (in the imap resource only)
-    //   imapRes.setOption( "Safety", "SSL"); // SSL/TLS
-    //   imapRes.setOption( "ImapPort", 993 );
-    // } else if ( arg == "tls" ) { // tls is really STARTTLS
-    //   imapRes.setOption( "Safety", "STARTTLS");  // STARTTLS
-    //   imapRes.setOption( "ImapPort", 143 );
-    // } else if ( arg == "none" ) {
-    //   imapRes.setOption( "Safety", "NONE" );  // No encryption
-    //   imapRes.setOption( "ImapPort", 143 );
-    // } else {
-    //   // safe default fallback when servertest failed
-    //   imapRes.setOption( "Safety", "STARTTLS");
-    //   imapRes.setOption( "ImapPort", 143 );
-    // }
-
     info.settings = settings;
     return info;
 }
@@ -166,59 +152,8 @@ Resource::ResourceInfo ManualConfiguration::createKolabResource() const
     settings.insert(QStringLiteral("SubscriptionEnabled"), true);
     settings.insert(QStringLiteral("Password"), mPassword);
     settings.insert(QStringLiteral("ImapPort"), mIncomingPort);
-
-    // TODO generate name
-    // TODO add setSettings(...)
+    info.settings = settings;
     info.typeIdentifier = QStringLiteral("akonadi_kolab_resource");
-#if 0
-    if (servertest_mode < 3) {   // submission & smtp
-        if (arg == "tls" ) { // tls is really STARTTLS
-          smtp.setEncryption("TLS");
-          if (servertest_mode == 1) {   //submission port 587
-              smtp.setPort(587);
-          } else {
-              smtp.setPort(25);
-          }
-        } else if ( arg == "ssl" ) {    //only possible as smtps
-            smtp.setPort(465);
-            smtp.setEncryption("SSL");
-        } else if (servertest_mode == 2) { //test submission and smtp failed or only possible unencrypted -> set to standard value and open editor
-            smtp.setPort(587);
-            smtp.setEncryption("TLS");
-            smtp.setEditMode(true);
-        } else if (servertest_mode == 1) { // submission test failed -> start smtp request
-            servertest_mode = 2;
-            ServerTest.test(page2.widget().lineEditSmtp.text, "smtp");
-            return;
-        }
-
-        // start imap test
-        servertest_mode = 3;
-        if (page2.widget().lineEditImap.text) {
-            SetupManager.setupInfo(qsTr("Probing IMAP server..."));
-            ServerTest.test(page2.widget().lineEditImap.text, "imap");
-        } else {
-            SetupManager.execute();
-        }
-    } else if (servertest_mode == 3) {   //imap
-        if ( arg == "ssl" ) {
-          // The ENUM used for authentication (in the kolab resource only)
-          kolabRes.setOption( "Safety", "SSL" ); // SSL/TLS
-          kolabRes.setOption( "ImapPort", 993 );
-        } else if ( arg == "tls" ) { // tls is really STARTTLS
-          kolabRes.setOption( "Safety", "STARTTLS" );  // STARTTLS
-          kolabRes.setOption( "ImapPort", 143 );
-        } else {
-          // safe default fallback in case server test failed
-          kolabRes.setOption( "Safety", "STARTTLS" );
-          kolabRes.setOption( "ImapPort", 143 );
-          kolabRes.setEditMode(true);
-        }
-        SetupManager.execute();
-    } else {
-        print ("Unknown servertest_mode = ", servertest_mode);
-    }
-#endif
     return info;
 }
 
@@ -241,6 +176,8 @@ void ManualConfiguration::createResource()
         qCWarning(ACCOUNTWIZARD_LOG) << " invalid protocol: " << mCurrentIncomingProtocol;
         return;
     }
+    // Debug it
+    qCDebug(ACCOUNTWIZARD_LOG) << " info " << info;
 
     auto resource = new Resource(this);
     resource->setResourceInfo(std::move(info));
@@ -268,7 +205,7 @@ void ManualConfiguration::setPassword(const QString &newPassword)
     mPassword = newPassword;
 }
 
-void ManualConfiguration::setIdentityId(uint id)
+void ManualConfiguration::setIdentityId(int id)
 {
     mIdentityId = id;
 }
@@ -307,16 +244,125 @@ QString ManualConfiguration::convertOutgoingSecurityProtocol(int protocol) const
 QString ManualConfiguration::convertOutgoingAuthenticationProtocol(int protocol) const
 {
     switch (protocol) {
-    case 0:
-        break;
+    case 0: // Clear Text
+        return QStringLiteral("clear");
+    case 1: // LOGIN
+        return QStringLiteral("login");
+    case 2: // PLAIN
+        return QStringLiteral("plain");
+    case 3: // CRAM-MD5
+        return QStringLiteral("cram-md5");
+    case 4: // DIGEST-MD5
+        return QStringLiteral("digest-md5");
+    case 5: // NTLM
+        return QStringLiteral("ntlm");
+    case 6: // GSSAPI
+        return QStringLiteral("gssapi");
     }
-    // TODO
+    qCWarning(ACCOUNTWIZARD_LOG) << " Impossible to convert protocol: " << protocol;
     return {};
 }
 
 void ManualConfiguration::checkServer()
 {
     qDebug() << " Verify server";
+    if (!mServerTest) {
+        mServerTest = new ServerTest(this);
+        connect(mServerTest, &ServerTest::testFail, this, &ManualConfiguration::slotTestFail);
+        connect(mServerTest, &ServerTest::testResult, this, &ManualConfiguration::slotTestResult);
+    }
+    QString protocol;
+    switch (mCurrentIncomingProtocol) {
+    case 0: // Pop3
+        protocol = QStringLiteral("pop");
+        break;
+    case 1: // Imap
+        protocol = QStringLiteral("imap");
+        break;
+    case 2: // Kolab
+        protocol = QStringLiteral("imap");
+        break;
+    }
+    // Test input
+    mServerTestInProgress = true;
+    Q_EMIT serverTestInProgressModeChanged();
+    mServerTest->test(mIncomingHostName, protocol);
+}
+
+void ManualConfiguration::slotTestFail()
+{
+    qDebug() << "slotTestFail  ";
+    // TODO
+    mServerTestInProgress = false;
+    Q_EMIT serverTestInProgressModeChanged();
+}
+
+void ManualConfiguration::slotTestResult(const QString &result)
+{
+    qDebug() << "slotTestResult  " << result;
+    switch (mCurrentIncomingProtocol) {
+    case 0: { // Pop3
+        if (result == QStringLiteral("ssl")) {
+            setIncomingPort(995);
+            // pop3Res.setOption( "UseTLS", true );
+        } else if (result == QStringLiteral("tls")) { // tls is really STARTTLS
+            setIncomingPort(110);
+            // pop3Res.setOption( "UseTLS", true );
+        } else if (result == QStringLiteral("none")) {
+            setIncomingPort(110);
+        } else {
+            setIncomingPort(110);
+        }
+        break;
+    }
+    case 1: { // Imap
+        if (result == QStringLiteral("ssl")) {
+            setIncomingPort(993);
+            // pop3Res.setOption( "UseTLS", true );
+        } else if (result == QStringLiteral("tls")) { // tls is really STARTTLS
+            setIncomingPort(143);
+            // pop3Res.setOption( "UseTLS", true );
+        } else if (result == QStringLiteral("none")) {
+            setIncomingPort(143);
+        } else {
+            setIncomingPort(143);
+        }
+        break;
+#if 0
+        // if ( server == "imap.gmail.com" ) {
+        //     imapRes.setOption( "Authentication", 9 ); // XOAuth2
+        //     arg = "ssl";
+        // } else {
+        //     imapRes.setOption( "Authentication", 7 ); // ClearText
+        // }
+        // if ( arg == "ssl" ) {
+        //   // The ENUM used for authentication (in the imap resource only)
+        //   imapRes.setOption( "Safety", "SSL"); // SSL/TLS
+        //   imapRes.setOption( "ImapPort", 993 );
+        // } else if ( arg == "tls" ) { // tls is really STARTTLS
+        //   imapRes.setOption( "Safety", "STARTTLS");  // STARTTLS
+        //   imapRes.setOption( "ImapPort", 143 );
+        // } else if ( arg == "none" ) {
+        //   imapRes.setOption( "Safety", "NONE" );  // No encryption
+        //   imapRes.setOption( "ImapPort", 143 );
+        // } else {
+        //   // safe default fallback when servertest failed
+        //   imapRes.setOption( "Safety", "STARTTLS");
+        //   imapRes.setOption( "ImapPort", 143 );
+        // }
+#endif
+        break;
+    }
+    }
+
+    // TODO
+    mServerTestInProgress = false;
+    Q_EMIT serverTestInProgressModeChanged();
+}
+
+int ManualConfiguration::identityId() const
+{
+    return mIdentityId;
 }
 
 void ManualConfiguration::checkConfiguration()
@@ -346,7 +392,7 @@ bool ManualConfiguration::disconnectedModeEnabled() const
     return mDisconnectedModeEnabled;
 }
 
-void ManualConfiguration::setDisconnectedModeEnabled(int disconnectedMode)
+void ManualConfiguration::setDisconnectedModeEnabled(bool disconnectedMode)
 {
     if (mDisconnectedModeEnabled == disconnectedMode)
         return;
@@ -453,12 +499,12 @@ void ManualConfiguration::setIncomingHostName(const QString &newIncomingHostName
     }
 }
 
-int ManualConfiguration::incomingPort() const
+uint ManualConfiguration::incomingPort() const
 {
     return mIncomingPort;
 }
 
-void ManualConfiguration::setIncomingPort(int newPort)
+void ManualConfiguration::setIncomingPort(uint newPort)
 {
     if (mIncomingPort != newPort) {
         mIncomingPort = newPort;
@@ -525,22 +571,23 @@ void ManualConfiguration::setOutgoingUserName(const QString &newOutgoingUserName
 
 QDebug operator<<(QDebug d, const ManualConfiguration &t)
 {
-    d << "mIncomingUserName " << t.incomingUserName();
-    d << "mIncomingHostName " << t.incomingHostName();
-    d << "mIncomingPort " << t.incomingPort();
+    d.space() << "mIncomingUserName" << t.incomingUserName();
+    d.space() << "mIncomingHostName" << t.incomingHostName();
+    d.space() << "mIncomingPort" << t.incomingPort();
 
-    d << "mOutgoingUserName " << t.outgoingUserName();
-    d << "mOutgoingHostName " << t.outgoingHostName();
-    d << "mOutgoingPort " << t.outgoingPort();
+    d.space() << "mOutgoingUserName" << t.outgoingUserName();
+    d.space() << "mOutgoingHostName" << t.outgoingHostName();
+    d.space() << "mOutgoingPort" << t.outgoingPort();
 
-    d << "mCurrentIncomingProtocol " << t.currentIncomingProtocol();
-    d << "mCurrentIncomingSecurityProtocol " << t.currentIncomingSecurityProtocol();
-    d << "mCurrentOutgoingSecurityProtocol " << t.currentOutgoingSecurityProtocol();
+    d.space() << "mCurrentIncomingProtocol" << t.currentIncomingProtocol();
+    d.space() << "mCurrentIncomingSecurityProtocol" << t.currentIncomingSecurityProtocol();
+    d.space() << "mCurrentOutgoingSecurityProtocol" << t.currentOutgoingSecurityProtocol();
 
-    d << "mCurrentIncomingAuthenticationProtocol " << t.currentIncomingAuthenticationProtocol();
-    d << "mCurrentOutgoingAuthenticationProtocol " << t.currentOutgoingAuthenticationProtocol();
+    d.space() << "mCurrentIncomingAuthenticationProtocol" << t.currentIncomingAuthenticationProtocol();
+    d.space() << "mCurrentOutgoingAuthenticationProtocol" << t.currentOutgoingAuthenticationProtocol();
 
-    d << "mDisconnectedModeEnabled " << t.disconnectedModeEnabled();
+    d.space() << "mDisconnectedModeEnabled" << t.disconnectedModeEnabled();
+    d.space() << "identity" << t.identityId();
     return d;
 }
 
