@@ -1,5 +1,5 @@
 /*
-    SPDX-FileCopyrightText: 2023 Laurent Montel <montel@kde.org>
+    SPDX-FileCopyrightText: 2023-2024 Laurent Montel <montel.org>
 
     SPDX-License-Identifier: LGPL-2.0-or-later
 */
@@ -10,12 +10,12 @@
 #include "transport.h"
 #include <QDebug>
 #include <QObject>
-
+class ServerTest;
 class LIBACCOUNTWIZARD_EXPORT ManualConfiguration : public QObject
 {
     Q_OBJECT
     Q_PROPERTY(QString incomingHostName READ incomingHostName WRITE setIncomingHostName NOTIFY incomingHostNameChanged FINAL)
-    Q_PROPERTY(int incomingPort READ incomingPort WRITE setIncomingPort NOTIFY incomingPortChanged FINAL)
+    Q_PROPERTY(uint incomingPort READ incomingPort WRITE setIncomingPort NOTIFY incomingPortChanged FINAL)
     Q_PROPERTY(QString incomingUserName READ incomingUserName WRITE setIncomingUserName NOTIFY incomingUserNameChanged FINAL)
 
     Q_PROPERTY(QString outgoingHostName READ outgoingHostName WRITE setOutgoingHostName NOTIFY outgoingHostNameChanged FINAL)
@@ -24,7 +24,7 @@ class LIBACCOUNTWIZARD_EXPORT ManualConfiguration : public QObject
     Q_PROPERTY(QStringList incomingProtocols READ incomingProtocols CONSTANT)
     Q_PROPERTY(QStringList securityProtocols READ securityProtocols CONSTANT)
     Q_PROPERTY(QStringList authenticationProtocols READ authenticationProtocols CONSTANT)
-    Q_PROPERTY(bool configurationIsValid MEMBER mConfigurationIsValid NOTIFY configurationIsValidChanged)
+    Q_PROPERTY(bool configurationIsValid MEMBER mConfigurationIsValid NOTIFY configurationIsValidChanged FINAL)
     Q_PROPERTY(int currentIncomingProtocol READ currentIncomingProtocol WRITE setCurrentIncomingProtocol NOTIFY currentIncomingProtocolChanged FINAL)
     Q_PROPERTY(int currentIncomingSecurityProtocol READ currentIncomingSecurityProtocol WRITE setCurrentIncomingSecurityProtocol NOTIFY
                    currentIncomingSecurityProtocolChanged FINAL)
@@ -37,7 +37,8 @@ class LIBACCOUNTWIZARD_EXPORT ManualConfiguration : public QObject
 
     Q_PROPERTY(bool disconnectedModeEnabled READ disconnectedModeEnabled WRITE setDisconnectedModeEnabled NOTIFY disconnectedModeEnabledChanged FINAL)
 
-    Q_PROPERTY(bool hasDisconnectedMode MEMBER mHasDisconnectedMode NOTIFY hasDisconnectedModeChanged)
+    Q_PROPERTY(bool hasDisconnectedMode MEMBER mHasDisconnectedMode NOTIFY hasDisconnectedModeChanged FINAL)
+    Q_PROPERTY(bool serverTestInProgress MEMBER mServerTestInProgress NOTIFY serverTestInProgressModeChanged FINAL)
 public:
     explicit ManualConfiguration(QObject *parent = nullptr);
     ~ManualConfiguration() override;
@@ -45,8 +46,8 @@ public:
     [[nodiscard]] QString incomingHostName() const;
     void setIncomingHostName(const QString &newIncomingHostName);
 
-    [[nodiscard]] int incomingPort() const;
-    void setIncomingPort(int newPort);
+    [[nodiscard]] uint incomingPort() const;
+    void setIncomingPort(uint newPort);
 
     [[nodiscard]] QString incomingUserName() const;
     void setIncomingUserName(const QString &newIncomingUserName);
@@ -80,7 +81,7 @@ public:
     void setCurrentOutgoingAuthenticationProtocol(int newCurrentOutgoingAuthenticationProtocols);
 
     [[nodiscard]] bool disconnectedModeEnabled() const;
-    void setDisconnectedModeEnabled(int disconnectedMode);
+    void setDisconnectedModeEnabled(bool disconnectedMode);
 
     Q_INVOKABLE void checkServer();
 
@@ -90,7 +91,9 @@ public:
 
     void setPassword(const QString &newPassword);
 
-    void setIdentityId(uint id);
+    void setIdentityId(int id);
+
+    int identityId() const;
 
 Q_SIGNALS:
     void incomingHostNameChanged();
@@ -118,6 +121,9 @@ Q_SIGNALS:
     void info(const QString &msg);
     void finished(const QString &msg);
 
+    void serverTestInProgressModeChanged();
+    void serverTestDone();
+
 private:
     [[nodiscard]] LIBACCOUNTWIZARD_NO_EXPORT Resource::ResourceInfo createPop3Resource() const;
     [[nodiscard]] LIBACCOUNTWIZARD_NO_EXPORT Resource::ResourceInfo createImapResource() const;
@@ -129,12 +135,15 @@ private:
     [[nodiscard]] LIBACCOUNTWIZARD_NO_EXPORT QString convertOutgoingSecurityProtocol(int protocol) const;
     [[nodiscard]] LIBACCOUNTWIZARD_NO_EXPORT QString convertOutgoingAuthenticationProtocol(int protocol) const;
     [[nodiscard]] LIBACCOUNTWIZARD_NO_EXPORT QString generateUniqueAccountName() const;
-    [[nodiscard]] LIBACCOUNTWIZARD_NO_EXPORT QString convertIncomingAuthenticationProtocol(int index) const;
+    [[nodiscard]] LIBACCOUNTWIZARD_NO_EXPORT QString convertIncomingAuthenticationProtocol(int protocol) const;
     [[nodiscard]] LIBACCOUNTWIZARD_NO_EXPORT QString convertIncomingSecurityProtocol(int index) const;
+    LIBACCOUNTWIZARD_NO_EXPORT void slotTestFail();
+    LIBACCOUNTWIZARD_NO_EXPORT void slotTestResult(const QString &result);
+
     // Incoming
     QString mIncomingUserName;
     QString mIncomingHostName;
-    int mIncomingPort = 995;
+    uint mIncomingPort = 995;
 
     // Outgoing
     QString mOutgoingUserName;
@@ -149,10 +158,10 @@ private:
     int mCurrentIncomingSecurityProtocol = 2; // NONE
     int mCurrentOutgoingSecurityProtocol = 2; // NONE
 
-    int mCurrentIncomingAuthenticationProtocol = 0;
-    int mCurrentOutgoingAuthenticationProtocol = 0;
+    int mCurrentIncomingAuthenticationProtocol = 0; // Clear Text
+    int mCurrentOutgoingAuthenticationProtocol = 0; // Clear Text
 
-    uint mIdentityId = 0;
+    int mIdentityId = 0;
 
     // configuration is Valid
     bool mConfigurationIsValid = false;
@@ -161,5 +170,9 @@ private:
     bool mHasDisconnectedMode = false;
 
     bool mDisconnectedModeEnabled = false;
+
+    bool mServerTestInProgress = false;
+
+    ServerTest *mServerTest = nullptr;
 };
 QDebug operator<<(QDebug d, const ManualConfiguration &t);
