@@ -1,5 +1,5 @@
 // SPDX-FileCopyrightText: 2023 Carl Schwan <carl@carlschwan.eu>
-// SPDX-FileCopyrightText: 2023 Laurent Montel <montel@kde.org>
+// SPDX-FileCopyrightText: 2023-2024 Laurent Montel <montel.org>
 // SPDX-License-Identifier: LGPL-2.0-or-later
 
 #include "setupmanager.h"
@@ -29,21 +29,37 @@ SetupManager::SetupManager(QObject *parent)
     connect(mManualConfiguration, &ManualConfiguration::info, this, &SetupManager::slotInfo);
 }
 
-SetupManager::~SetupManager() = default;
+SetupManager::~SetupManager()
+{
+    if (!mAccountCreated) {
+        mIdentity->destroy();
+    }
+}
 
 void SetupManager::slotError(const QString &str)
 {
-    // TODO
+    mDetails += str + QLatin1Char('\n');
+    // TODO add color ?
+    Q_EMIT detailsChanged();
 }
 
 void SetupManager::slotFinished(const QString &str)
 {
-    // TODO
+    mDetails += str + QLatin1Char('\n');
+    // TODO add color ?
+    Q_EMIT detailsChanged();
 }
 
 void SetupManager::slotInfo(const QString &str)
 {
-    // TODO
+    mDetails += str + QLatin1Char('\n');
+    // TODO add color ?
+    Q_EMIT detailsChanged();
+}
+
+QString SetupManager::details() const
+{
+    return mDetails;
 }
 
 QString SetupManager::fullName() const
@@ -103,6 +119,12 @@ void SetupManager::searchConfiguration()
 void SetupManager::createAutomaticAccount()
 {
     qCDebug(ACCOUNTWIZARD_LOG) << " Create Automatic Account";
+    mIdentity->create();
+    const uint id = mIdentity->uoid();
+    // TODO use password
+    // TODO use selected account
+
+    mAccountCreated = true;
 }
 
 void SetupManager::createManualAccount()
@@ -112,6 +134,7 @@ void SetupManager::createManualAccount()
     const uint id = mIdentity->uoid();
     mManualConfiguration->setIdentityId(id);
     mManualConfiguration->createManualAccount();
+    mAccountCreated = true;
 }
 
 void SetupManager::setEmailProvider(const EmailProvider &emailProvider, const QString &messageInfo)
@@ -129,6 +152,7 @@ void SetupManager::clearConfiguration()
     Q_EMIT searchIspdbFoundMessageChanged();
 
     mNoConfigFound = false;
+    mAccountCreated = false;
     Q_EMIT noConfigFoundChanged();
 }
 
